@@ -752,14 +752,16 @@ reset_size_callback (void)
     g_assert_true (data_2.destroyed);
 }
 
+#ifdef HAVE_PIXBUF
 static void
 zero_size_func (gint *width, gint *height, gpointer user_data)
 {
+    (void) user_data;
+
     *width = 0;
     *height = 0;
 }
 
-#ifdef HAVE_PIXBUF
 static void
 render_with_zero_size_callback (void)
 {
@@ -792,6 +794,8 @@ render_with_zero_size_callback (void)
 static void
 pixbuf_size_func (gint *width, gint *height, gpointer user_data)
 {
+    (void) user_data;
+
     *width = 420;
     *height = 430;
 }
@@ -1169,6 +1173,32 @@ render_layer (void)
     g_object_unref (handle);
 }
 
+static void
+set_cancellable_for_rendering (void)
+{
+    RsvgHandle *handle = load_test_document ("layers.svg");
+
+    cairo_surface_t *output = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 300, 300);
+    cairo_t *cr = cairo_create (output);
+
+    RsvgRectangle viewport = { 100.0, 100.0, 100.0, 100.0 };
+
+    GError *error = NULL;
+
+    /* Same as in the Rust API test, we cancel immediately and then start rendering. */
+    GCancellable *cancellable = g_cancellable_new ();
+    g_cancellable_cancel (cancellable);
+
+    rsvg_handle_set_cancellable_for_rendering (handle, cancellable);
+
+    g_assert_false (rsvg_handle_render_layer (handle, cr, "#bar", &viewport, &error));
+    g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+
+    cairo_destroy (cr);
+    cairo_surface_destroy (output);
+    g_object_unref (handle);
+}
+
 static void
 untransformed_element (void)
 {
@@ -1585,8 +1615,9 @@ typedef struct
 } DimensionsFixtureData;
 
 static void
-test_dimensions (DimensionsFixtureData *fixture)
+test_dimensions (gconstpointer user_data)
 {
+    const DimensionsFixtureData *fixture = user_data;
     RsvgHandle *handle;
     RsvgPositionData position;
     RsvgDimensionData dimension;
@@ -1727,7 +1758,7 @@ static LoadingTestData loading_tests[] = {
 static void
 add_pixbuf_tests (void)
 {
-    int i;
+    gsize i;
 
     /* Tests for rsvg_handle_get_pixbuf() and rsvg_handle_get_pixbuf_sub() */
     g_test_add_func ("/api/handle_get_pixbuf", handle_get_pixbuf);
@@ -1766,7 +1797,9 @@ add_api_tests (void)
     g_test_add_func ("/api/dimensions_and_position", dimensions_and_position);
     g_test_add_func ("/api/set_size_callback", set_size_callback);
     g_test_add_func ("/api/reset_size_callback", reset_size_callback);
+#ifdef HAVE_PIXBUF
     g_test_add_func ("/api/render_with_zero_size_callback", render_with_zero_size_callback);
+#endif
     g_test_add_func ("/api/detects_cairo_context_in_error", detects_cairo_context_in_error);
     g_test_add_func ("/api/can_draw_to_non_image_surface", can_draw_to_non_image_surface);
     g_test_add_func ("/api/render_cairo_sub", render_cairo_sub);
@@ -1778,6 +1811,7 @@ add_api_tests (void)
     g_test_add_func ("/api/render_document", render_document);
     g_test_add_func ("/api/get_geometry_for_layer", get_geometry_for_layer);
     g_test_add_func ("/api/render_layer", render_layer);
+    g_test_add_func ("/api/set_cancellable_for_rendering", set_cancellable_for_rendering);
     g_test_add_func ("/api/untransformed_element", untransformed_element);
     g_test_add_func ("/api/no_write_before_close", no_write_before_close);
     g_test_add_func ("/api/empty_write_close", empty_write_close);
@@ -1803,17 +1837,17 @@ add_api_tests (void)
 static void
 add_geometry_tests (void)
 {
-    int i;
+    gsize i;
 
     for (i = 0; i < G_N_ELEMENTS (dimensions_fixtures); i++)
-        g_test_add_data_func (dimensions_fixtures[i].test_name, &dimensions_fixtures[i], (void*)test_dimensions);
+        g_test_add_data_func (dimensions_fixtures[i].test_name, &dimensions_fixtures[i], test_dimensions);
 }
 
 /* Tests for the deprecated API for loading bytes at a time */
 static void
 add_loading_tests (void)
 {
-    int i;
+    gsize i;
 
     for (i = 0; i < G_N_ELEMENTS (loading_tests); i++) {
         g_test_add_data_func (loading_tests[i].test_name, &loading_tests[i], load_n_bytes_at_a_time);
