@@ -27,6 +27,7 @@ use crate::is_element_of_type;
 use crate::limits;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeData};
 use crate::rect::Rect;
+use crate::rsvg_log;
 use crate::session::Session;
 use crate::structure::IntrinsicDimensions;
 use crate::surface_utils::shared_surface::SharedImageSurface;
@@ -214,13 +215,17 @@ impl Document {
     }
 
     /// Looks up a node in this document or one of its resources by its `id` attribute.
-    fn lookup_node(&self, node_id: &NodeId) -> Option<Node> {
+    fn lookup_node(
+        &self,
+        node_id: &NodeId,
+        cancellable: Option<&gio::Cancellable>,
+    ) -> Option<Node> {
         match node_id {
             NodeId::Internal(id) => self.lookup_internal_node(id),
             NodeId::External(url, id) => self
                 .resources
                 .borrow_mut()
-                .lookup_node(&self.session, &self.load_options, url, id)
+                .lookup_node(&self.session, &self.load_options, url, id, cancellable)
                 .ok(),
         }
     }
@@ -230,33 +235,24 @@ impl Document {
         self.ids.get(id).map(|n| (*n).clone())
     }
 
-    /// Loads an image by URL, or returns a pre-loaded one.
-    fn lookup_image(&self, url: &str) -> Result<SharedImageSurface, LoadingError> {
-        let aurl = self
-            .load_options
-            .url_resolver
-            .resolve_href(url)
-            .map_err(|_| LoadingError::BadUrl)?;
-
-        self.resources
-            .borrow_mut()
-            .lookup_image(&self.session, &self.load_options, &aurl)
-    }
-
     /// Loads a resource by URL, or returns a pre-loaded one.
-    fn lookup_resource(&self, url: &str) -> Result<Resource, LoadingError> {
+    fn lookup_resource(
+        &self,
+        url: &str,
+        cancellable: Option<&gio::Cancellable>,
+    ) -> Result<Resource, LoadingError> {
         let aurl = self
             .load_options
             .url_resolver
             .resolve_href(url)
             .map_err(|_| LoadingError::BadUrl)?;
 
-        // FIXME: pass a cancellable to this.  This function is called
-        // at rendering time, so probably the cancellable should come
-        // from cancellability in CairoRenderer - see #429
-        self.resources
-            .borrow_mut()
-            .lookup_resource(&self.session, &self.load_options, &aurl, None)
+        self.resources.borrow_mut().lookup_resource(
+            &self.session,
+            &self.load_options,
+            &aurl,
+            cancellable,
+        )
     }
 
     /// Runs the CSS cascade on the document tree
@@ -326,7 +322,7 @@ impl Document {
                 cr,
                 viewport,
                 config,
-                &mut AcquiredNodes::new(self),
+                &mut AcquiredNodes::new(self, options.cancellable.clone()),
             )
             .map(|_bbox| ())
         })
@@ -352,7 +348,7 @@ impl Document {
             &cr,
             viewport,
             config,
-            &mut AcquiredNodes::new(self),
+            &mut AcquiredNodes::new(self, options.cancellable.clone()),
         )?;
 
         let ink_rect = bbox.ink_rect.unwrap_or_default();
@@ -397,7 +393,7 @@ impl Document {
             &cr,
             unit_rectangle(),
             config,
-            &mut AcquiredNodes::new(self),
+            &mut AcquiredNodes::new(self, options.cancellable.clone()),
         )
     }
 
@@ -463,7 +459,7 @@ impl Document {
                 cr,
                 unit_rectangle(),
                 config,
-                &mut AcquiredNodes::new(self),
+                &mut AcquiredNodes::new(self, options.cancellable.clone()),
             )
             .map(|_bbox| ())
         })
@@ -498,8 +494,9 @@ impl Resources {
         load_options: &LoadOptions,
         url: &str,
         id: &str,
+        cancellable: Option<&gio::Cancellable>,
     ) -> Result<Node, LoadingError> {
-        self.get_extern_document(session, load_options, url)
+        self.get_extern_document(session, load_options, url, cancellable)
             .and_then(|resource| match resource {
                 Resource::Document(doc) => doc.lookup_internal_node(id).ok_or(LoadingError::BadUrl),
                 _ => unreachable!("get_extern_document() should already have ensured the document"),
@@ -511,16 +508,14 @@ impl Resources {
         session: &Session,
         load_options: &LoadOptions,
         href: &str,
+        cancellable: Option<&gio::Cancellable>,
     ) -> Result<Resource, LoadingError> {
         let aurl = load_options
             .url_resolver
             .resolve_href(href)
             .map_err(|_| LoadingError::BadUrl)?;
 
-        // FIXME: pass a cancellable to this.  This function is called
-        // at rendering time, so probably the cancellable should come
-        // from cancellability in CairoRenderer - see #429
-        let resource = self.lookup_resource(session, load_options, &aurl, None)?;
+        let resource = self.lookup_resource(session, load_options, &aurl, cancellable)?;
 
         match resource {
             Resource::Document(_) => Ok(resource),
@@ -530,23 +525,6 @@ impl Resources {
         }
     }
 
-    fn lookup_image(
-        &mut self,
-        session: &Session,
-        load_options: &LoadOptions,
-        aurl: &AllowedUrl,
-    ) -> Result<SharedImageSurface, LoadingError> {
-        // FIXME: pass a cancellable to this.  This function is called
-        // at rendering time, so probably the cancellable should come
-        // from cancellability in CairoRenderer - see #429
-        let resource = self.lookup_resource(session, load_options, aurl, None)?;
-
-        match resource {
-            Resource::Image(image) => Ok(image),
-            _ => Err(LoadingError::Other(format!("{aurl} is not a raster image"))),
-        }
-    }
-
     fn lookup_resource(
         &mut self,
         session: &Session,
@@ -767,24 +745,23 @@ pub struct AcquiredNodes<'i> {
     num_elements_acquired: usize,
     node_stack: Rc<RefCell<NodeStack>>,
     nodes_with_cycles: Vec<Node>,
+    cancellable: Option<gio::Cancellable>,
 }
 
 impl<'i> AcquiredNodes<'i> {
-    pub fn new(document: &Document) -> AcquiredNodes<'_> {
+    pub fn new(document: &Document, cancellable: Option<gio::Cancellable>) -> AcquiredNodes<'_> {
         AcquiredNodes {
             document,
             num_elements_acquired: 0,
             node_stack: Rc::new(RefCell::new(NodeStack::new())),
             nodes_with_cycles: Vec::new(),
+            cancellable,
         }
     }
 
-    pub fn lookup_image(&self, href: &str) -> Result<SharedImageSurface, LoadingError> {
-        self.document.lookup_image(href)
-    }
-
     pub fn lookup_resource(&self, url: &str) -> Result<Resource, LoadingError> {
-        self.document.lookup_resource(url)
+        self.document
+            .lookup_resource(url, self.cancellable.as_ref())
     }
 
     /// Acquires a node by its id.
@@ -809,7 +786,7 @@ impl<'i> AcquiredNodes<'i> {
         //   - Now that all files are loaded, resolve URL references
         let node = self
             .document
-            .lookup_node(node_id)
+            .lookup_node(node_id, self.cancellable.as_ref())
             .ok_or_else(|| AcquireError::LinkNotFound(node_id.clone()))?;
 
         if self.nodes_with_cycles.contains(&node) {
@@ -830,11 +807,11 @@ impl<'i> AcquiredNodes<'i> {
     /// needs to be re-acquired with this method.  For example:
     ///
     /// * At an "early resolution" stage, `acquire()` a pattern by its id, and keep around its
-    /// [`Node`] reference.
+    ///   [`Node`] reference.
     ///
     /// * At the drawing stage, `acquire_ref()` the pattern node that we already had, so that
-    /// its child elements that reference other paint servers will be able to detect circular
-    /// references to the pattern.
+    ///   its child elements that reference other paint servers will be able to detect circular
+    ///   references to the pattern.
     pub fn acquire_ref(&mut self, node: &Node) -> Result<AcquiredNode, AcquireError> {
         if self.nodes_with_cycles.contains(node) {
             Err(AcquireError::CircularReference(node.clone()))
@@ -942,10 +919,15 @@ impl DocumentBuilder {
         let node = Node::new(NodeData::new_element(&self.session, name, attrs));
 
         if let Some(id) = node.borrow_element().get_id() {
-            // This is so we don't overwrite an existing id
-            self.ids
-                .entry(id.to_string())
-                .or_insert_with(|| node.clone());
+            match self.ids.entry(id.to_string()) {
+                Entry::Occupied(_) => {
+                    rsvg_log!(self.session, "ignoring duplicate id {id} for {node}");
+                }
+
+                Entry::Vacant(e) => {
+                    e.insert(node.clone());
+                }
+            }
         }
 
         if let Some(parent) = parent {
