@@ -8,12 +8,12 @@ use pango::prelude::FontMapExt;
 use regex::{Captures, Regex};
 use std::cell::RefCell;
 use std::convert::TryFrom;
-use std::f64::consts::*;
 use std::rc::Rc;
 use std::{borrow::Cow, sync::OnceLock};
 
 use crate::accept_language::UserLanguage;
 use crate::bbox::BoundingBox;
+use crate::cairo_path::CairoPath;
 use crate::color::color_to_rgba;
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId, RenderingOptions};
@@ -28,10 +28,10 @@ use crate::layout::{
     Text, TextSpan,
 };
 use crate::length::*;
+use crate::limits;
 use crate::marker;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::paint_server::{PaintSource, UserSpacePaintSource};
-use crate::path_builder::*;
 use crate::pattern::UserSpacePattern;
 use crate::properties::{
     ClipRule, ComputedValues, FillRule, ImageRendering, MaskType, MixBlendMode, Opacity,
@@ -61,8 +61,7 @@ pub struct FontOptions {
 struct PathHelper<'a> {
     cr: &'a cairo::Context,
     transform: ValidTransform,
-    path: &'a Path,
-    is_square_linecap: bool,
+    cairo_path: &'a CairoPath,
     has_path: Option<bool>,
 }
 
@@ -70,14 +69,12 @@ impl<'a> PathHelper<'a> {
     pub fn new(
         cr: &'a cairo::Context,
         transform: ValidTransform,
-        path: &'a Path,
-        linecap: StrokeLinecap,
+        cairo_path: &'a CairoPath,
     ) -> Self {
         PathHelper {
             cr,
             transform,
-            path,
-            is_square_linecap: linecap == StrokeLinecap::Square,
+            cairo_path,
             has_path: None,
         }
     }
@@ -87,7 +84,7 @@ impl<'a> PathHelper<'a> {
             Some(false) | None => {
                 self.has_path = Some(true);
                 self.cr.set_matrix(self.transform.into());
-                self.path.to_cairo(self.cr, self.is_square_linecap)
+                self.cairo_path.to_cairo_context(self.cr)
             }
             Some(true) => Ok(()),
         }
@@ -193,6 +190,12 @@ pub struct DrawingCtx {
     drawsub_stack: Vec<Node>,
 
     config: RenderingConfiguration,
+
+    /// Depth of nested layers while drawing.
+    ///
+    /// We use this to set a hard limit on how many nested layers there can be, to avoid
+    /// malicious SVGs that would cause unbounded stack consumption.
+    recursion_depth: u16,
 }
 
 pub enum DrawingMode {
@@ -326,6 +329,7 @@ impl DrawingCtx {
             cr: cr.clone(),
             drawsub_stack,
             config,
+            recursion_depth: 0,
         }
     }
 
@@ -335,19 +339,20 @@ impl DrawingCtx {
     /// would like to use that same state but on a different Cairo surface and context
     /// than the ones being used on `self`.  This function copies the `self` state into a
     /// new `DrawingCtx`, and ties the copied one to the supplied `cr`.
-    fn nested(&self, cr: cairo::Context) -> DrawingCtx {
+    fn nested(&self, cr: cairo::Context) -> Box<DrawingCtx> {
         let cr_stack = self.cr_stack.clone();
 
         cr_stack.borrow_mut().push(self.cr.clone());
 
-        DrawingCtx {
+        Box::new(DrawingCtx {
             session: self.session.clone(),
             initial_viewport: self.initial_viewport.clone(),
             cr_stack,
             cr,
             drawsub_stack: self.drawsub_stack.clone(),
             config: self.config.clone(),
-        }
+            recursion_depth: self.recursion_depth,
+        })
     }
 
     pub fn session(&self) -> &Session {
@@ -634,14 +639,14 @@ impl DrawingCtx {
 
             let mut mask_draw_ctx = self.nested(mask_cr);
 
-            let stacking_ctx = StackingContext::new(
+            let stacking_ctx = Box::new(StackingContext::new(
                 self.session(),
                 acquired_nodes,
                 &mask_element,
                 Transform::identity(),
                 None,
                 values,
-            );
+            ));
 
             rsvg_log!(self.session, "(mask {}", mask_element);
 
@@ -680,7 +685,110 @@ impl DrawingCtx {
         }
     }
 
-    pub fn with_discrete_layer(
+    /// Checks whether the rendering has been cancelled in the middle.
+    ///
+    /// If so, returns an Err.  This is used from [`DrawingCtx::with_discrete_layer`] to
+    /// exit early instead of proceeding with rendering.
+    fn check_cancellation(&self) -> Result<(), InternalRenderingError> {
+        if self.is_rendering_cancelled() {
+            return Err(InternalRenderingError::Cancelled);
+        }
+
+        Ok(())
+    }
+
+    fn check_layer_nesting_depth(&mut self) -> Result<(), InternalRenderingError> {
+        if self.recursion_depth > limits::MAX_LAYER_NESTING_DEPTH {
+            return Err(InternalRenderingError::LimitExceeded(
+                ImplementationLimit::MaximumLayerNestingDepthExceeded,
+            ));
+        }
+
+        Ok(())
+    }
+
+    fn filter_current_surface(
+        &mut self,
+        acquired_nodes: &mut AcquiredNodes<'_>,
+        filter: &Filter,
+        viewport: &Viewport,
+        element_name: &str,
+        bbox: &BoundingBox,
+    ) -> Result<cairo::Surface, InternalRenderingError> {
+        let surface_to_filter = SharedImageSurface::copy_from_surface(
+            &cairo::ImageSurface::try_from(self.cr.target()).unwrap(),
+        )?;
+
+        let stroke_paint_source = Rc::new(filter.stroke_paint_source.to_user_space(
+            &bbox.rect,
+            viewport,
+            &filter.normalize_values,
+        ));
+        let fill_paint_source = Rc::new(filter.fill_paint_source.to_user_space(
+            &bbox.rect,
+            viewport,
+            &filter.normalize_values,
+        ));
+
+        // Filter functions (like "blend()", not the <filter> element) require
+        // being resolved in userSpaceonUse units, since that is the default
+        // for primitive_units.  So, get the corresponding NormalizeParams
+        // here and pass them down.
+        let user_space_params = NormalizeParams::from_values(
+            &filter.normalize_values,
+            &viewport.with_units(CoordUnits::UserSpaceOnUse),
+        );
+
+        let filtered_surface = self
+            .run_filters(
+                viewport,
+                surface_to_filter,
+                filter,
+                acquired_nodes,
+                element_name,
+                &user_space_params,
+                stroke_paint_source,
+                fill_paint_source,
+                bbox,
+            )?
+            .into_image_surface()?;
+
+        let generic_surface: &cairo::Surface = &filtered_surface; // deref to Surface
+
+        Ok(generic_surface.clone())
+    }
+
+    fn draw_in_optional_new_viewport(
+        &mut self,
+        acquired_nodes: &mut AcquiredNodes<'_>,
+        viewport: &Viewport,
+        layout_viewport: &Option<LayoutViewport>,
+        draw_fn: &mut dyn FnMut(
+            &mut AcquiredNodes<'_>,
+            &mut DrawingCtx,
+            &Viewport,
+        ) -> Result<BoundingBox, InternalRenderingError>,
+    ) -> Result<BoundingBox, InternalRenderingError> {
+        if let Some(layout_viewport) = layout_viewport.as_ref() {
+            // FIXME: here we ignore the Some() result of push_new_viewport().  We do that because
+            // the returned one is just a copy of the one that got passeed in, but with a changed
+            // transform.  However, we are in fact not using that transform anywhere!
+            //
+            // In case push_new_viewport() returns None, we just don't draw anything.
+            //
+            // Note that push_new_viewport() changes the cr's transform.  However it will be restored
+            // at the end of this function with set_matrix.
+            if let Some(new_viewport) = self.push_new_viewport(viewport, layout_viewport) {
+                draw_fn(acquired_nodes, self, &new_viewport)
+            } else {
+                Ok(self.empty_bbox())
+            }
+        } else {
+            draw_fn(acquired_nodes, self, viewport)
+        }
+    }
+
+    fn draw_layer_internal(
         &mut self,
         stacking_ctx: &StackingContext,
         acquired_nodes: &mut AcquiredNodes<'_>,
@@ -693,36 +801,16 @@ impl DrawingCtx {
             &Viewport,
         ) -> Result<BoundingBox, InternalRenderingError>,
     ) -> Result<BoundingBox, InternalRenderingError> {
-        if self.is_rendering_cancelled() {
-            return Err(InternalRenderingError::Cancelled);
-        }
-
         let stacking_ctx_transform = ValidTransform::try_from(stacking_ctx.transform)?;
 
         let orig_transform = self.get_transform();
 
-        // See the comment below about "not using that transform anywhere" (the viewport's).
+        // See the comment above about "not using that transform anywhere" (the viewport's).
         let viewport = viewport.with_composed_transform(stacking_ctx.transform);
         self.cr.transform(stacking_ctx_transform.into());
 
         let res = if clipping {
-            if let Some(layout_viewport) = layout_viewport.as_ref() {
-                // FIXME: here we ignore the Some() result of push_new_viewport().  We do that because
-                // the returned one is just a copy of the one that got passeed in, but with a changed
-                // transform.  However, we are in fact not using that transform anywhere!
-                //
-                // In case push_new_viewport() returns None, we just don't draw anything.
-                //
-                // Note that push_new_viewport() changes the cr's transform.  However it will be restored
-                // at the end of this function with set_matrix.
-                if let Some(new_viewport) = self.push_new_viewport(&viewport, layout_viewport) {
-                    draw_fn(acquired_nodes, self, &new_viewport)
-                } else {
-                    Ok(self.empty_bbox())
-                }
-            } else {
-                draw_fn(acquired_nodes, self, &viewport)
-            }
+            self.draw_in_optional_new_viewport(acquired_nodes, &viewport, &layout_viewport, draw_fn)
         } else {
             with_saved_cr(&self.cr.clone(), || {
                 if let Some(ref link_target) = stacking_ctx.link_target {
@@ -750,11 +838,11 @@ impl DrawingCtx {
                 let res = if should_isolate {
                     // Compute our assortment of affines
 
-                    let affines = CompositingAffines::new(
+                    let affines = Box::new(CompositingAffines::new(
                         *affine_at_start,
                         self.initial_viewport.transform,
                         self.cr_stack.borrow().len(),
-                    );
+                    ));
 
                     // Create temporary surface and its cr
 
@@ -776,22 +864,12 @@ impl DrawingCtx {
                         // Draw!
 
                         let res = with_saved_cr(&cr, || {
-                            if let Some(layout_viewport) = layout_viewport.as_ref() {
-                                // FIXME: here we ignore the Some() result of push_new_viewport().  We do that because
-                                // the returned one is just a copy of the one that got passeed in, but with a changed
-                                // transform.  However, we are in fact not using that transform anywhere!
-                                //
-                                // In case push_new_viewport() returns None, we just don't draw anything.
-                                if let Some(new_viewport) =
-                                    temporary_draw_ctx.push_new_viewport(&viewport, layout_viewport)
-                                {
-                                    draw_fn(acquired_nodes, &mut temporary_draw_ctx, &new_viewport)
-                                } else {
-                                    Ok(self.empty_bbox())
-                                }
-                            } else {
-                                draw_fn(acquired_nodes, &mut temporary_draw_ctx, &viewport)
-                            }
+                            temporary_draw_ctx.draw_in_optional_new_viewport(
+                                acquired_nodes,
+                                &viewport,
+                                &layout_viewport,
+                                draw_fn,
+                            )
                         });
 
                         let bbox = if let Ok(ref bbox) = res {
@@ -801,50 +879,19 @@ impl DrawingCtx {
                         };
 
                         if let Some(ref filter) = stacking_ctx.filter {
-                            let surface_to_filter = SharedImageSurface::copy_from_surface(
-                                &cairo::ImageSurface::try_from(temporary_draw_ctx.cr.target())
-                                    .unwrap(),
+                            let filtered_surface = temporary_draw_ctx.filter_current_surface(
+                                acquired_nodes,
+                                filter,
+                                &viewport,
+                                &stacking_ctx.element_name,
+                                &bbox,
                             )?;
 
-                            let stroke_paint_source =
-                                Rc::new(filter.stroke_paint_source.to_user_space(
-                                    &bbox.rect,
-                                    &viewport,
-                                    &filter.normalize_values,
-                                ));
-                            let fill_paint_source =
-                                Rc::new(filter.fill_paint_source.to_user_space(
-                                    &bbox.rect,
-                                    &viewport,
-                                    &filter.normalize_values,
-                                ));
-
-                            // Filter functions (like "blend()", not the <filter> element) require
-                            // being resolved in userSpaceonUse units, since that is the default
-                            // for primitive_units.  So, get the corresponding NormalizeParams
-                            // here and pass them down.
-                            let user_space_params = NormalizeParams::from_values(
-                                &filter.normalize_values,
-                                &viewport.with_units(CoordUnits::UserSpaceOnUse),
-                            );
-
-                            let filtered_surface = temporary_draw_ctx
-                                .run_filters(
-                                    &viewport,
-                                    surface_to_filter,
-                                    filter,
-                                    acquired_nodes,
-                                    &stacking_ctx.element_name,
-                                    &user_space_params,
-                                    stroke_paint_source,
-                                    fill_paint_source,
-                                    bbox,
-                                )?
-                                .into_image_surface()?;
-
-                            let generic_surface: &cairo::Surface = &filtered_surface; // deref to Surface
-
-                            (generic_surface.clone(), res, bbox)
+                            // FIXME: "res" was declared mutable above so that we could overwrite it
+                            // with the result of filtering, so that if filtering produces an error,
+                            // then the masking below wouldn't take place.  Test for that and fix this;
+                            // we are *not* modifying res in case of error.
+                            (filtered_surface, res, bbox)
                         } else {
                             (temporary_draw_ctx.cr.target(), res, bbox)
                         }
@@ -913,23 +960,13 @@ impl DrawingCtx {
 
                     self.cr.set_matrix(affine_at_start.into());
                     res
-                } else if let Some(layout_viewport) = layout_viewport.as_ref() {
-                    // FIXME: here we ignore the Some() result of push_new_viewport().  We do that because
-                    // the returned one is just a copy of the one that got passeed in, but with a changed
-                    // transform.  However, we are in fact not using that transform anywhere!
-                    //
-                    // In case push_new_viewport() returns None, we just don't draw anything.
-                    //
-                    // Note that push_new_viewport() changes the cr's transform.  However it will be restored
-                    // at the end of this function with set_matrix.
-                    if let Some(new_viewport) = self.push_new_viewport(&viewport, layout_viewport) {
-                        draw_fn(acquired_nodes, self, &new_viewport)
-                    } else {
-                        self.cr.set_matrix(orig_transform.into());
-                        Ok(self.empty_bbox())
-                    }
                 } else {
-                    draw_fn(acquired_nodes, self, &viewport)
+                    self.draw_in_optional_new_viewport(
+                        acquired_nodes,
+                        &viewport,
+                        &layout_viewport,
+                        draw_fn,
+                    )
                 };
 
                 if stacking_ctx.link_target.is_some() {
@@ -944,6 +981,42 @@ impl DrawingCtx {
         res
     }
 
+    pub fn with_discrete_layer(
+        &mut self,
+        stacking_ctx: &StackingContext,
+        acquired_nodes: &mut AcquiredNodes<'_>,
+        viewport: &Viewport,
+        layout_viewport: Option<LayoutViewport>,
+        clipping: bool,
+        draw_fn: &mut dyn FnMut(
+            &mut AcquiredNodes<'_>,
+            &mut DrawingCtx,
+            &Viewport,
+        ) -> Result<BoundingBox, InternalRenderingError>,
+    ) -> Result<BoundingBox, InternalRenderingError> {
+        self.check_cancellation()?;
+
+        self.recursion_depth += 1;
+
+        match self.check_layer_nesting_depth() {
+            Ok(()) => {
+                let res = self.draw_layer_internal(
+                    stacking_ctx,
+                    acquired_nodes,
+                    viewport,
+                    layout_viewport,
+                    clipping,
+                    draw_fn,
+                );
+
+                self.recursion_depth -= 1;
+                res
+            }
+
+            Err(e) => Err(e),
+        }
+    }
+
     /// Run the drawing function with the specified opacity
     fn with_alpha(
         &mut self,
@@ -987,7 +1060,7 @@ impl DrawingCtx {
         user_space_params: &NormalizeParams,
         stroke_paint_source: Rc<UserSpacePaintSource>,
         fill_paint_source: Rc<UserSpacePaintSource>,
-        node_bbox: BoundingBox,
+        node_bbox: &BoundingBox,
     ) -> Result<SharedImageSurface, InternalRenderingError> {
         let session = self.session();
 
@@ -1163,14 +1236,14 @@ impl DrawingCtx {
 
                     let elt = pattern_node.borrow_element();
 
-                    let stacking_ctx = StackingContext::new(
+                    let stacking_ctx = Box::new(StackingContext::new(
                         self.session(),
                         acquired_nodes,
                         &elt,
                         Transform::identity(),
                         None,
                         pattern_values,
-                    );
+                    ));
 
                     dc.with_discrete_layer(
                         &stacking_ctx,
@@ -1335,18 +1408,15 @@ impl DrawingCtx {
         clipping: bool,
         viewport: &Viewport,
     ) -> Result<BoundingBox, InternalRenderingError> {
-        let (path, stroke_paint, fill_paint) = match &shape.path {
+        let (cairo_path, stroke_paint, fill_paint) = match &shape.path {
             layout::Path::Validated {
-                path,
+                cairo_path,
                 extents: Some(_),
                 stroke_paint,
                 fill_paint,
-            } => (path, stroke_paint, fill_paint),
-            layout::Path::Validated {
-                path: _,
-                extents: None,
                 ..
-            } => return Ok(self.empty_bbox()),
+            } => (cairo_path, stroke_paint, fill_paint),
+            layout::Path::Validated { extents: None, .. } => return Ok(self.empty_bbox()),
             layout::Path::Invalid(_) => return Ok(self.empty_bbox()),
         };
 
@@ -1360,7 +1430,7 @@ impl DrawingCtx {
                 let cr = dc.cr.clone();
 
                 let transform = dc.get_transform_for_stacking_ctx(stacking_ctx, clipping)?;
-                let mut path_helper = PathHelper::new(&cr, transform, path, shape.stroke.line_cap);
+                let mut path_helper = PathHelper::new(&cr, transform, cairo_path);
 
                 if clipping {
                     if shape.is_visible {
@@ -1531,7 +1601,7 @@ impl DrawingCtx {
         acquired_nodes: &mut AcquiredNodes<'_>,
         clipping: bool,
     ) -> Result<BoundingBox, InternalRenderingError> {
-        let path = pango_layout_to_path(span.x, span.y, &span.layout, span.gravity)?;
+        let path = pango_layout_to_cairo_path(span.x, span.y, &span.layout, span.gravity)?;
         if path.is_empty() {
             // Empty strings, or only-whitespace text, get turned into empty paths.
             // In that case, we really want to return "no bounds" rather than an
@@ -1551,11 +1621,11 @@ impl DrawingCtx {
             setup_cr_for_stroke(&self.cr, &span.stroke);
 
             if clipping {
-                path.to_cairo(&self.cr, false)?;
+                path.to_cairo_context(&self.cr)?;
                 return Ok(self.empty_bbox());
             }
 
-            path.to_cairo(&self.cr, false)?;
+            path.to_cairo_context(&self.cr)?;
             let bbox = compute_stroke_and_fill_box(
                 &self.cr,
                 &span.stroke,
@@ -1577,7 +1647,7 @@ impl DrawingCtx {
 
                             if had_paint_server {
                                 if can_use_text_as_path {
-                                    path.to_cairo(&self.cr, false)?;
+                                    path.to_cairo_context(&self.cr)?;
                                     self.cr.fill()?;
                                     self.cr.new_path();
                                 } else {
@@ -1603,7 +1673,7 @@ impl DrawingCtx {
                                 self.set_paint_source(&span.stroke_paint, acquired_nodes)?;
 
                             if had_paint_server {
-                                path.to_cairo(&self.cr, false)?;
+                                path.to_cairo_context(&self.cr)?;
                                 self.cr.stroke()?;
                                 self.cr.new_path();
                             }
@@ -1856,14 +1926,14 @@ impl DrawingCtx {
 
             let child_values = elt.get_computed_values();
 
-            let stacking_ctx = StackingContext::new(
+            let stacking_ctx = Box::new(StackingContext::new(
                 self.session(),
                 acquired_nodes,
                 &use_element,
                 Transform::identity(),
                 None,
                 values,
-            );
+            ));
 
             let layout_viewport = LayoutViewport {
                 vbox,
@@ -1896,14 +1966,14 @@ impl DrawingCtx {
         } else {
             // otherwise the referenced node is not a <symbol>; process it generically
 
-            let stacking_ctx = StackingContext::new(
+            let stacking_ctx = Box::new(StackingContext::new(
                 self.session(),
                 acquired_nodes,
                 &use_element,
                 Transform::new_translate(use_rect.x0, use_rect.y0),
                 None,
                 values,
-            );
+            ));
 
             self.with_discrete_layer(
                 &stacking_ctx,
@@ -1970,20 +2040,6 @@ impl From<ImageRendering> for Interpolation {
     }
 }
 
-pub fn compute_path_extents(path: &Path) -> Result<Option<Rect>, InternalRenderingError> {
-    if path.is_empty() {
-        return Ok(None);
-    }
-
-    let surface = cairo::RecordingSurface::create(cairo::Content::ColorAlpha, None)?;
-    let cr = cairo::Context::new(&surface)?;
-
-    path.to_cairo(&cr, false)?;
-    let (x0, y0, x1, y1) = cr.path_extents()?;
-
-    Ok(Some(Rect::new(x0, y0, x1, y1)))
-}
-
 /// Create a Pango context with a particular configuration.
 pub fn create_pango_context(font_options: &FontOptions, transform: &Transform) -> pango::Context {
     let font_map = pangocairo::FontMap::default();
@@ -2067,20 +2123,20 @@ fn pango_layout_to_cairo(
     cr.set_matrix(matrix);
 }
 
-/// Converts a Pango layout to a Path starting at (x, y).
-pub fn pango_layout_to_path(
+/// Converts a Pango layout to a CairoPath starting at (x, y).
+fn pango_layout_to_cairo_path(
     x: f64,
     y: f64,
     layout: &pango::Layout,
     gravity: pango::Gravity,
-) -> Result<Path, InternalRenderingError> {
+) -> Result<CairoPath, InternalRenderingError> {
     let surface = cairo::RecordingSurface::create(cairo::Content::ColorAlpha, None)?;
     let cr = cairo::Context::new(&surface)?;
 
     pango_layout_to_cairo(x, y, layout, gravity, &cr);
 
     let cairo_path = cr.copy_path()?;
-    Ok(Path::from_cairo(cairo_path))
+    Ok(CairoPath::from_cairo(cairo_path))
 }
 
 // https://www.w3.org/TR/css-masking-1/#ClipPathElement
@@ -2420,151 +2476,3 @@ pub struct PathExtents {
     /// Extents for the stroked path, or `None` if the path is empty or zero-width.
     pub stroke: Option<Rect>,
 }
-
-impl Path {
-    pub fn to_cairo(
-        &self,
-        cr: &cairo::Context,
-        is_square_linecap: bool,
-    ) -> Result<(), InternalRenderingError> {
-        assert!(!self.is_empty());
-
-        for subpath in self.iter_subpath() {
-            // If a subpath is empty and the linecap is a square, then draw a square centered on
-            // the origin of the subpath. See #165.
-            if is_square_linecap {
-                let (x, y) = subpath.origin();
-                if subpath.is_zero_length() {
-                    let stroke_size = 0.002;
-
-                    cr.move_to(x - stroke_size / 2., y);
-                    cr.line_to(x + stroke_size / 2., y);
-                }
-            }
-
-            for cmd in subpath.iter_commands() {
-                cmd.to_cairo(cr);
-            }
-        }
-
-        // We check the cr's status right after feeding it a new path for a few reasons:
-        //
-        // * Any of the individual path commands may cause the cr to enter an error state, for
-        //   example, if they come with coordinates outside of Cairo's supported range.
-        //
-        // * The *next* call to the cr will probably be something that actually checks the status
-        //   (i.e. in cairo-rs), and we don't want to panic there.
-
-        cr.status().map_err(|e| e.into())
-    }
-
-    /// Converts a `cairo::Path` to a librsvg `Path`.
-    fn from_cairo(cairo_path: cairo::Path) -> Path {
-        let mut builder = PathBuilder::default();
-
-        // Cairo has the habit of appending a MoveTo to some paths, but we don't want a
-        // path for empty text to generate that lone point.  So, strip out paths composed
-        // only of MoveTo.
-
-        if !cairo_path_is_only_move_tos(&cairo_path) {
-            for segment in cairo_path.iter() {
-                match segment {
-                    cairo::PathSegment::MoveTo((x, y)) => builder.move_to(x, y),
-                    cairo::PathSegment::LineTo((x, y)) => builder.line_to(x, y),
-                    cairo::PathSegment::CurveTo((x2, y2), (x3, y3), (x4, y4)) => {
-                        builder.curve_to(x2, y2, x3, y3, x4, y4)
-                    }
-                    cairo::PathSegment::ClosePath => builder.close_path(),
-                }
-            }
-        }
-
-        builder.into_path()
-    }
-}
-
-fn cairo_path_is_only_move_tos(path: &cairo::Path) -> bool {
-    path.iter()
-        .all(|seg| matches!(seg, cairo::PathSegment::MoveTo((_, _))))
-}
-
-impl PathCommand {
-    fn to_cairo(&self, cr: &cairo::Context) {
-        match *self {
-            PathCommand::MoveTo(x, y) => cr.move_to(x, y),
-            PathCommand::LineTo(x, y) => cr.line_to(x, y),
-            PathCommand::CurveTo(ref curve) => curve.to_cairo(cr),
-            PathCommand::Arc(ref arc) => arc.to_cairo(cr),
-            PathCommand::ClosePath => cr.close_path(),
-        }
-    }
-}
-
-impl EllipticalArc {
-    fn to_cairo(&self, cr: &cairo::Context) {
-        match self.center_parameterization() {
-            ArcParameterization::CenterParameters {
-                center,
-                radii,
-                theta1,
-                delta_theta,
-            } => {
-                let n_segs = (delta_theta / (PI * 0.5 + 0.001)).abs().ceil() as u32;
-                let d_theta = delta_theta / f64::from(n_segs);
-
-                let mut theta = theta1;
-                for _ in 0..n_segs {
-                    arc_segment(center, radii, self.x_axis_rotation, theta, theta + d_theta)
-                        .to_cairo(cr);
-                    theta += d_theta;
-                }
-            }
-            ArcParameterization::LineTo => {
-                let (x2, y2) = self.to;
-                cr.line_to(x2, y2);
-            }
-            ArcParameterization::Omit => {}
-        }
-    }
-}
-
-impl CubicBezierCurve {
-    fn to_cairo(&self, cr: &cairo::Context) {
-        let Self { pt1, pt2, to } = *self;
-        cr.curve_to(pt1.0, pt1.1, pt2.0, pt2.1, to.0, to.1);
-    }
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn rsvg_path_from_cairo_path() {
-        let surface = cairo::ImageSurface::create(cairo::Format::ARgb32, 10, 10).unwrap();
-        let cr = cairo::Context::new(&surface).unwrap();
-
-        cr.move_to(1.0, 2.0);
-        cr.line_to(3.0, 4.0);
-        cr.curve_to(5.0, 6.0, 7.0, 8.0, 9.0, 10.0);
-        cr.close_path();
-
-        let cairo_path = cr.copy_path().unwrap();
-        let path = Path::from_cairo(cairo_path);
-
-        assert_eq!(
-            path.iter().collect::<Vec<PathCommand>>(),
-            vec![
-                PathCommand::MoveTo(1.0, 2.0),
-                PathCommand::LineTo(3.0, 4.0),
-                PathCommand::CurveTo(CubicBezierCurve {
-                    pt1: (5.0, 6.0),
-                    pt2: (7.0, 8.0),
-                    to: (9.0, 10.0),
-                }),
-                PathCommand::ClosePath,
-                PathCommand::MoveTo(1.0, 2.0), // cairo inserts a MoveTo after ClosePath
-            ],
-        );
-    }
-}
