@@ -6,6 +6,53 @@ use float_cmp::ApproxEq;
 // cairo-fixed-type-private.h}
 
 const CAIRO_FIXED_FRAC_BITS: u64 = 8;
+const CAIRO_MAGIC_NUMBER_FIXED: f64 = (1u64 << (52 - CAIRO_FIXED_FRAC_BITS)) as f64 * 1.5;
+
+const CAIRO_FIXED_MAX: i32 = i32::MAX;
+const CAIRO_FIXED_MIN: i32 = i32::MIN;
+
+/// The double that corresponds to (the number one in fixed-point representation)
+const CAIRO_FIXED_ONE_DOUBLE: f64 = (1 << CAIRO_FIXED_FRAC_BITS) as f64;
+
+/// The largest representable fixed-point number, as a double.  This is a bit over 8 million.
+pub const CAIRO_FIXED_MAX_DOUBLE: f64 = (CAIRO_FIXED_MAX as f64) / CAIRO_FIXED_ONE_DOUBLE;
+
+/// The most negative representable fixed-point number, as a double.  This is a bit less than -8 million.
+pub const CAIRO_FIXED_MIN_DOUBLE: f64 = (CAIRO_FIXED_MIN as f64) / CAIRO_FIXED_ONE_DOUBLE;
+
+fn cairo_magic_double(d: f64) -> f64 {
+    d + CAIRO_MAGIC_NUMBER_FIXED
+}
+
+fn cairo_fixed_from_double(d: f64) -> i32 {
+    let bits = cairo_magic_double(d).to_bits();
+    let lower = bits & 0xffffffff;
+    lower as i32
+}
+
+/// Implements a method to check whether two `f64` numbers would have
+/// the same fixed-point representation in Cairo.
+///
+/// This generally means that the absolute difference between them,
+/// when taken as floating-point numbers, is less than the smallest
+/// representable fraction that Cairo can represent in fixed-point.
+///
+/// Implementation detail: Cairo fixed-point numbers use 24 bits for
+/// the integral part, and 8 bits for the fractional part.  That is,
+/// the smallest fraction they can represent is 1/256.
+pub trait FixedEqCairo {
+    #[allow(dead_code)] // https://github.com/rust-lang/rust/issues/120770
+    fn fixed_eq_cairo(&self, other: &Self) -> bool;
+}
+
+impl FixedEqCairo for f64 {
+    fn fixed_eq_cairo(&self, other: &f64) -> bool {
+        // FIXME: Here we have the same problem as Cairo itself: we
+        // don't check for overflow in the conversion of double to
+        // fixed-point.
+        cairo_fixed_from_double(*self) == cairo_fixed_from_double(*other)
+    }
+}
 
 /// Checks whether two floating-point numbers are approximately equal,
 /// considering Cairo's limitations on numeric representation.
@@ -55,6 +102,15 @@ macro_rules! assert_approx_eq_cairo {
 mod tests {
     use super::*;
 
+    #[test]
+    fn numbers_equal_in_cairo_fixed_point() {
+        assert!(1.0_f64.fixed_eq_cairo(&1.0_f64));
+
+        assert!(1.0_f64.fixed_eq_cairo(&1.001953125_f64)); // 1 + 1/512 - cairo rounds to 1
+
+        assert!(!1.0_f64.fixed_eq_cairo(&1.00390625_f64)); // 1 + 1/256 - cairo can represent it
+    }
+
     #[test]
     fn numbers_approx_equal() {
         // 0 == 1/256 - cairo can represent it, so not equal
