@@ -1,6 +1,7 @@
 //! Text elements: `text`, `tspan`, `tref`.
 
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
+use markup5ever::{expanded_name, local_name, namespace_url, ns, QualName};
+use pango::prelude::FontExt;
 use pango::IsAttribute;
 use std::cell::RefCell;
 use std::convert::TryFrom;
@@ -15,10 +16,10 @@ use crate::layout::{self, FontProperties, Layer, LayerKind, StackingContext, Str
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::paint_server::PaintSource;
-use crate::parsers::ParseValue;
+use crate::parsers::{CommaSeparatedList, Parse, ParseValue};
 use crate::properties::{
-    ComputedValues, Direction, FontStretch, FontStyle, FontVariant, FontWeight, PaintOrder,
-    TextAnchor, TextRendering, UnicodeBidi, WritingMode, XmlLang, XmlSpace,
+    ComputedValues, Direction, DominantBaseline, FontStretch, FontStyle, FontVariant, FontWeight,
+    PaintOrder, TextAnchor, TextRendering, UnicodeBidi, WritingMode, XmlLang, XmlSpace,
 };
 use crate::rect::Rect;
 use crate::rsvg_log;
@@ -287,8 +288,65 @@ fn compute_baseline_offset(
     values: &ComputedValues,
     params: &NormalizeParams,
 ) -> f64 {
-    let baseline = f64::from(layout.baseline()) / f64::from(pango::SCALE);
+    let mut baseline = f64::from(layout.baseline()) / f64::from(pango::SCALE);
+    let dominant_baseline = values.dominant_baseline();
+
+    let mut layout_iter = layout.iter();
+    loop {
+        let run = layout_iter.run_readonly();
+
+        if run.is_some() {
+            let item = run.unwrap().item();
+            let font = item.analysis().font();
+
+            let metrics = font.metrics(None);
+            let ascent = metrics.ascent();
+            let descent = metrics.descent();
+            let height = metrics.height();
+
+            match dominant_baseline {
+                DominantBaseline::Hanging => {
+                    baseline -= f64::from(ascent - descent) / f64::from(pango::SCALE);
+                }
+                DominantBaseline::Middle => {
+                    // Approximate meanline using strikethrough position and thickness
+                    // https://mail.gnome.org/archives/gtk-i18n-list/2012-December/msg00046.html
+                    baseline -= f64::from(
+                        metrics.strikethrough_position() + metrics.strikethrough_thickness() / 2,
+                    ) / f64::from(pango::SCALE);
+                }
+                DominantBaseline::Central => {
+                    baseline = 0.5 * f64::from(ascent + descent) / f64::from(pango::SCALE);
+                }
+                DominantBaseline::TextBeforeEdge | DominantBaseline::TextTop => {
+                    //baseline -= f64::from(ascent) / f64::from(pango::SCALE);
+                    // Bit of a klutch, but leads to better results
+                    baseline -= f64::from(2 * ascent - height) / f64::from(pango::SCALE);
+                }
+                DominantBaseline::TextAfterEdge | DominantBaseline::TextBottom => {
+                    baseline += f64::from(descent) / f64::from(pango::SCALE);
+                }
+                DominantBaseline::Ideographic => {
+                    // Approx
+                    baseline += f64::from(descent) / f64::from(pango::SCALE);
+                }
+                DominantBaseline::Mathematical => {
+                    // Approx
+                    baseline = 0.5 * f64::from(ascent + descent) / f64::from(pango::SCALE);
+                }
+                _ => (),
+            }
+
+            break;
+        }
+
+        if !layout_iter.next_run() {
+            break;
+        }
+    }
+
     let baseline_shift = values.baseline_shift().0.to_user(params);
+
     baseline + baseline_shift
 }
 
@@ -753,14 +811,44 @@ impl Text {
     }
 }
 
+// Parse an (optionally) comma-separated list and just return the first element.
+//
+// From https://gitlab.gnome.org/GNOME/librsvg/-/issues/183, the current implementation
+// of text layout only supports a single value for the x/y/dx/dy attributes.  However,
+// we need to be able to parse values with multiple lengths.  So, we'll do that, but just
+// use the first value from each attribute.
+fn parse_list_and_extract_first<T: Copy + Default + Parse>(
+    dest: &mut T,
+    attr: QualName,
+    value: &str,
+    session: &Session,
+) {
+    let mut list: CommaSeparatedList<T, 0, 1024> = CommaSeparatedList(Vec::new());
+
+    set_attribute(&mut list, attr.parse(value), session);
+    if list.0.is_empty() {
+        *dest = Default::default();
+    } else {
+        *dest = list.0[0]; // ignore all but the first element
+    }
+}
+
 impl ElementTrait for Text {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => set_attribute(&mut self.x, attr.parse(value), session),
-                expanded_name!("", "y") => set_attribute(&mut self.y, attr.parse(value), session),
-                expanded_name!("", "dx") => set_attribute(&mut self.dx, attr.parse(value), session),
-                expanded_name!("", "dy") => set_attribute(&mut self.dy, attr.parse(value), session),
+                expanded_name!("", "x") => {
+                    parse_list_and_extract_first(&mut self.x, attr, value, session)
+                }
+                expanded_name!("", "y") => {
+                    parse_list_and_extract_first(&mut self.y, attr, value, session)
+                }
+                expanded_name!("", "dx") => {
+                    parse_list_and_extract_first(&mut self.dx, attr, value, session)
+                }
+                expanded_name!("", "dy") => {
+                    parse_list_and_extract_first(&mut self.dy, attr, value, session)
+                }
                 _ => (),
             }
         }
@@ -1028,10 +1116,18 @@ impl ElementTrait for TSpan {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => set_attribute(&mut self.x, attr.parse(value), session),
-                expanded_name!("", "y") => set_attribute(&mut self.y, attr.parse(value), session),
-                expanded_name!("", "dx") => set_attribute(&mut self.dx, attr.parse(value), session),
-                expanded_name!("", "dy") => set_attribute(&mut self.dy, attr.parse(value), session),
+                expanded_name!("", "x") => {
+                    parse_list_and_extract_first(&mut self.x, attr, value, session)
+                }
+                expanded_name!("", "y") => {
+                    parse_list_and_extract_first(&mut self.y, attr, value, session)
+                }
+                expanded_name!("", "dx") => {
+                    parse_list_and_extract_first(&mut self.dx, attr, value, session)
+                }
+                expanded_name!("", "dy") => {
+                    parse_list_and_extract_first(&mut self.dy, attr, value, session)
+                }
                 _ => (),
             }
         }
